﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Able_Opus_Multi_Downloader.Properties;
using BSF_Code_Library.Helpers;

namespace Able_Opus_Multi_Downloader
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            Application.ApplicationExit +=new EventHandler(Application_ApplicationExit);
            AutoSetDownloadDir();
            InitializeComponent();
            chkAutoClear.Checked = Settings.Default.AutoClear;
            chkShowErrorLog.Checked = Settings.Default.ShowErrorLog;
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            base.OnKeyDown(e);

            // Delete selected downloads
            if (e.KeyCode.Equals(Keys.Delete))
            {
                foreach (DownloadBar bar in pnlActiveDownloads.SelectedDownloads)
                {
                    bar.StopDownload();
                }
            }
            else if (KeyState.IsKeyPressed(Keys.Up) && KeyState.IsKeyPressed(Keys.ControlKey))
            {
                if (pnlActiveDownloads.Controls.Count.Equals(0)) return;

                int lastSelected = -1;

                DownloadBar[] bars = pnlActiveDownloads.DownloadBars;

                for (int i = bars.Length - 1; i > 0; i--)
                {
                    if (bars[i].IsSelected)
                    {
                        lastSelected = i;
                        break;
                    }
                }

                if (!KeyState.IsKeyPressed(Keys.ShiftKey))
                {
                    pnlActiveDownloads.DeSelectAllDownloads();
                }

                if (lastSelected == -1)
                {
                    pnlActiveDownloads.SelectItem(bars.Length - 1);
                    pnlActiveDownloads.ScrollControlIntoView(bars[bars.Length - 1]);
                }
                else if (lastSelected + 1 < bars.Length)
                {
                    pnlActiveDownloads.SelectItem(lastSelected + 1);
                    pnlActiveDownloads.ScrollControlIntoView(bars[lastSelected + 1]);
                }
                else
                {
                    pnlActiveDownloads.SelectItem(0);
                    pnlActiveDownloads.ScrollControlIntoView(bars[0]);
                }
            }
            else if (KeyState.IsKeyPressed(Keys.Down) && KeyState.IsKeyPressed(Keys.ControlKey))
            {
                if (pnlActiveDownloads.Controls.Count.Equals(0)) return;

                int lastSelected = -1;

                DownloadBar[] bars = pnlActiveDownloads.DownloadBars;

                for (int i = 0; i < bars.Length; i++)
                {
                    if (bars[i].IsSelected)
                    {
                        lastSelected = i;
                        break;
                    }
                }

                if (!KeyState.IsKeyPressed(Keys.ShiftKey))
                {
                    pnlActiveDownloads.DeSelectAllDownloads();
                }

                if (lastSelected == -1)
                {
                    pnlActiveDownloads.SelectItem(0);
                    pnlActiveDownloads.ScrollControlIntoView(bars[0]);
                }
                else if (lastSelected - 1 >= 0)
                {
                    pnlActiveDownloads.SelectItem(lastSelected - 1);
                    pnlActiveDownloads.ScrollControlIntoView(bars[lastSelected - 1]);
                }
                else
                {
                    pnlActiveDownloads.SelectItem(bars.Length - 1);
                    pnlActiveDownloads.ScrollControlIntoView(bars[bars.Length - 1]);
                }
            }
            else if (e.KeyCode == Keys.C && e.Control)
            {
                DownloadBar[] bars = pnlActiveDownloads.SelectedDownloads;

                if (bars.Length > 0)
                {
                    Clipboard.SetText(bars[0].DownloadLocation);
                }
            }
        }

        private void Application_ApplicationExit(object sender, EventArgs e)
        {
             Settings.Default.Save();
             pnlActiveDownloads.StopAndClearAllDownloads();
        }

        private static void AutoSetDownloadDir()
        {
            if (Settings.Default.DownloadDir.Length == 0)
            {
                string userPath = Environment.GetFolderPath(Environment.SpecialFolder.UserProfile);

                if (Directory.Exists(userPath + "\\Downloads"))
                {
                    Settings.Default.DownloadDir = userPath + "\\Downloads";
                }
                else if (Directory.Exists(userPath + "\\My Downloads"))
                {
                    Settings.Default.DownloadDir = userPath + "\\My Downloads";
                }
            }
        }

        private void btnClearInactive_Click(object sender, EventArgs e)
        {
            pnlActiveDownloads.ClearAllInactiveDownloads();
        }

        private void btnAddDownload_Click(object sender, EventArgs e)
        {
            pnlActiveDownloads.AddDownload(txtAddDownload.Text);
        }

        private void btnDownloadClipboard_Click(object sender, EventArgs e)
        {
            pnlActiveDownloads.AddDownload(Clipboard.GetText());
        }

        private void btnClearAll_Click(object sender, EventArgs e)
        {
            pnlActiveDownloads.StopAndClearAllDownloads();
        }

        private void btnClearSelected_Click(object sender, EventArgs e)
        {
            pnlActiveDownloads.StopAndClearSelected();
        }

        private void btnDownloadClipboard_MouseEnter(object sender, EventArgs e)
        {
            string toolTipText = btnDownloadClipboard.Tag.ToString() + "\n\n" + Clipboard.GetText();

            if (toolTipText != toolTip.GetToolTip(btnDownloadClipboard))
            {
                toolTip.SetToolTip(btnDownloadClipboard, toolTipText);
            }
        }

        private void txtAddDownload_TextChanged(object sender, EventArgs e)
        {
            txtFilename.Text = Path.GetFileName(txtAddDownload.Text);
            btnAddDownload.Enabled = (txtAddDownload.TextLength > 0) && (txtFilename.TextLength > 0);
        }

        private void txtFilename_TextChanged(object sender, EventArgs e)
        {
            btnAddDownload.Enabled = (txtAddDownload.TextLength > 0) && (txtFilename.TextLength > 0);
        }

        private void btnPageDownload_Click(object sender, EventArgs e)
        {
            using (WebpageMinerForm frmPageMiner = new WebpageMinerForm())
            {
                if (frmPageMiner.ShowDialog() == DialogResult.OK)
                {
                    pnlActiveDownloads.AddDownloadRange(frmPageMiner.LinksFound);
                }
            }
        }

        private void pnlActiveDownloads_SelectedChanged(object sender, EventArgs e)
        {
            DownloadBar[] bars = pnlActiveDownloads.SelectedDownloads;

            if (bars.Length > 0)
            {
                lblFullPath.Text = lblFullPath.Tag + " " + bars[0].DownloadLocation;
            }
        }

        private void pnlActiveDownloads_ControlRemoved(object sender, ControlEventArgs e)
        {
            if (pnlActiveDownloads.SelectedDownloads.Length.Equals(0))
            {
                lblFullPath.Text = string.Empty;
            }
        }

        private void tmrUpdate_Tick(object sender, EventArgs e)
        {
            if (!pnlActiveDownloads.AllDownloadsCompleted)
            {
                int totalProgress = pnlActiveDownloads.TotalProgress;
                this.Text = totalProgress.ToString() + "% Complete";
            }
            else
            {
                tmrUpdate.Stop();

                if (notifyIcon.Visible)
                {
                    notifyIcon.ShowBalloonTip(5, "Done!", "All downloads completed!", ToolTipIcon.Info);
                }
                else
                {
                    MessageBox.Show("All downloads completed!", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }

                this.Text = Application.ProductName;

                if (Misc.HasError && Settings.Default.ShowErrorLog)
                {
                    Misc.ShowErrorLog();
                }

                return;
            }
        }

        private void pnlActiveDownloads_DownloadAdded(object sender, EventArgs e)
        {
            tmrUpdate.Start();
        }

        private void chkAutoClear_CheckedChanged(object sender, EventArgs e)
        {
            Settings.Default.AutoClear = chkAutoClear.Checked;
        }

        private void btnMinimizeToTray_Paint(object sender, PaintEventArgs e)
        {
            // Draw Down arrow
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;

            e.Graphics.FillPolygon(Brushes.Black, new Point[] 
            {
                new Point(6, 6), 
                new Point(btnMinimizeToTray.Width - 6, 6), 
                new Point(btnMinimizeToTray.Width / 2, btnMinimizeToTray.Height - 8 )});
        }

        private void btnMinimizeToTray_Click(object sender, EventArgs e)
        {
            this.Hide();
            notifyIcon.Visible = true;
        }

        private void notifyIcon_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            this.Show();
            notifyIcon.Visible = false;
        }

        private void tsmiExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void tsmiClear_Click(object sender, EventArgs e)
        {
            pnlActiveDownloads.StopAndClearAllDownloads();
        }

        private void tsmiShow_Click(object sender, EventArgs e)
        {
            this.Show();
            notifyIcon.Visible = false;
        }
    }
}
