﻿using System;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;

namespace Able_Opus_Multi_Downloader
{
    class RolloverButton : Button
    {
        private Image _mouseEnterImage, _mouseLeaveImage;

        public RolloverButton()
        {
            this.Cursor = Cursors.Hand;
            this.FlatAppearance.BorderSize = 0;
            this.BackColor = Color.Transparent;
            this.FlatStyle = FlatStyle.Flat;
            this.FlatAppearance.MouseDownBackColor = Color.Transparent;
            this.FlatAppearance.MouseOverBackColor = Color.Transparent;
        }

        /// <summary>
        /// Hide selection rectangle
        /// </summary>
        protected override bool ShowFocusCues
        {
            get { return false; }
        }

        protected override void OnPaint(PaintEventArgs pevent)
        {
            base.OnPaint(pevent);

            if (this.Parent != null)
            {
                ControlPaint.DrawBorder(pevent.Graphics, this.ClientRectangle,
                    this.Parent.BackColor, ButtonBorderStyle.Solid);
            }
        }

        protected override void OnMouseEnter(EventArgs e)
        {
            if (_mouseEnterImage != null)
            {
                this.Image = _mouseEnterImage;
            }

            base.OnMouseEnter(e);
        }

        protected override void OnMouseLeave(EventArgs e)
        {
            if (_mouseLeaveImage != null)
            {
                this.Image = _mouseLeaveImage;
            }

            base.OnMouseEnter(e);
        }

        [Description("The image to display when the cursor enters the control")]
        [Category("Effects")]
        public Image MouseEnterImage
        {
            set { _mouseEnterImage = value; }
            get { return _mouseEnterImage; }
        }

        [Description("The image to display when the cursor leaves the control")]
        [Category("Effects")]
        public Image MouseLeaveImage
        {
            set 
            { 
                _mouseLeaveImage = value;
                this.Image = value;
                this.Text = string.Empty;
            }
            get { return _mouseLeaveImage; }
        }
    }
}
