﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Drawing2D;

namespace BSFControls.UserControls
{
    public partial class SuperProgressBar : Control
    {
        private float _progress = 0f;
        private bool _showPercent = false;
        private Graphics _backBuffer;
        private Bitmap _drawingSurface;
        private Brush _foreBrush;
        private SolidBrush _progressBrush = new SolidBrush(Color.YellowGreen);

        public SuperProgressBar()
        {
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.ResizeRedraw, true);
            _foreBrush = new SolidBrush(this.ForeColor);
            this.MinimumSize = new Size(10, 10);
            this.Size = new Size(200, 23);
            this.BackColor = Color.YellowGreen;
        }

        static public Color ChangeColorBrightness(Color color, int factor)
        {
            int R = (color.R + factor > 255) ? 255 : color.R + factor;
            int G = (color.G + factor > 255) ? 255 : color.G + factor;
            int B = (color.B + factor > 255) ? 255 : color.B + factor;

            return Color.FromArgb(R, G, B);
        }

        private void UpdateBackBuffer()
        {
            _drawingSurface = new Bitmap(this.Width, this.Height);
            _backBuffer = Graphics.FromImage(_drawingSurface);
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            //Draw grayness
            _backBuffer.FillRectangle(Brushes.LightGray, this.ClientRectangle);
            //Draw progress
            _backBuffer.FillRectangle(_progressBrush, 0, 0, this.Width * (_progress / 100f), this.Height);

            if (_showPercent)
            {
                string text = ((int)(_progress + 0.5)).ToString() + "%";
                float strWidth = _backBuffer.MeasureString(text, this.Font).Width;
                float strHeight = _backBuffer.MeasureString(text, this.Font).Height;

                float xPos = this.Width / 2 - strWidth / 2;
                float yPos = this.Height / 2 - strHeight / 2;
                PointF p1 = new PointF(xPos, yPos);
                _backBuffer.DrawString(text, this.Font, _foreBrush, p1);
            }

            e.Graphics.DrawImageUnscaled(_drawingSurface, Point.Empty);
        }

        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            UpdateBackBuffer();
        }

        /// <summary>
        /// Gets or swets the value ranging from 0-100 which represents progress
        /// </summary>
        [Description("The value ranging from 0-100 which represents progress")]
        public float Progress
        {
            get { return _progress; }
            set
            {
                if (value > 100)
                {
                    _progress = 100;
                }
                else if (value < 0)
                {
                    _progress = 0;
                }
                else
                {
                    _progress = value;
                }

                this.Invalidate();
            }
        }

        /// <summary>
        /// Determines whether or not to show the numeric percentage
        /// </summary>
        [Description("Determines whether or not to show the numeric percentage")]
        [Category("Appearance")]
        public bool ShowPercentage
        {
            get { return _showPercent; }
            set 
            { 
                _showPercent = value;
                this.Invalidate();
            }
        }

        [DefaultValue(typeof(Color), "YellowGreen")]
        public override Color BackColor
        {
            get { return base.BackColor; }
            set
            {
                base.BackColor = value;
                _progressBrush = new SolidBrush(value);

                if (this != null)
                {
                    this.Invalidate();
                }
            }
        }

        public override Color ForeColor
        {
            get { return base.ForeColor; }
            set
            {
                base.ForeColor = value;
                _foreBrush = new SolidBrush(this.ForeColor);
                this.Invalidate();
            }
        }

        public override Font Font
        {
            get { return base.Font; }
            set
            {
                base.Font = value;
                this.Invalidate();
            }
        }
    }
}

