﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Able_Opus_Multi_Downloader.Properties;
using System.Net;
using System.Text.RegularExpressions;
using System.IO;

namespace Able_Opus_Multi_Downloader
{
    public partial class WebpageMinerForm : Form
    {
        private List<string> _linksFound = new List<string>();

        public WebpageMinerForm()
        {
            InitializeComponent();
            LoadSettingsAndDefaults();
        }

        private void LoadSettingsAndDefaults()
        {
            txtAdress.Text = Settings.Default.LastWebpage;
            chkFilterFilename.Checked = Settings.Default.IsFilteringName;
            txtName.Text = Settings.Default.FilterName;
            chkHref.Checked = Settings.Default.HrefAttrib;
            chkSrc.Checked = Settings.Default.SrcAttrib;
            chkMatchCase.Checked = Settings.Default.MatchCase;
            chkSearchFull.Checked = Settings.Default.SearchFullPathForTerm;
            chkUseCustom.Checked = Settings.Default.UseCustomRegex;
        }

        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);

            Settings.Default.LastWebpage = txtAdress.Text;
            Settings.Default.IsFilteringName = chkFilterFilename.Checked;
            Settings.Default.FilterName = txtName.Text;
            Settings.Default.HrefAttrib = chkHref.Checked;
            Settings.Default.SrcAttrib = chkSrc.Checked;
            Settings.Default.MatchCase = chkMatchCase.Checked;
            Settings.Default.SearchFullPathForTerm = chkSearchFull.Checked;
            Settings.Default.UseCustomRegex = chkUseCustom.Checked;
        }

        private void txtAdress_TextChanged(object sender, EventArgs e)
        {
            btnBegin.Enabled = (txtAdress.TextLength > 0);
        }

        private void btnPickURL_Click(object sender, EventArgs e)
        {
            using (PickURLForm frmPickUrl = new PickURLForm())
            {
                if (frmPickUrl.ShowDialog() == DialogResult.OK)
                {
                    txtAdress.Text = frmPickUrl.SelectedURL;
                }
            }
        }

        private void FixWebPageAdress()
        {
            txtAdress.Text = txtAdress.Text.Trim();
            const string PATTERN = @"(https*|ftp)://";
            bool isOK = Regex.IsMatch(txtAdress.Text, PATTERN, RegexOptions.IgnoreCase);

            if (!isOK)
            {
                txtAdress.Text = "http://" + txtAdress.Text;
            }
        }

        private void btnBegin_Click(object sender, EventArgs e)
        {
            FixWebPageAdress();
            this.Text = "Downloading Page...";
            this.Enabled = false;
            WebClient webClient = new WebClient();
            webClient.DownloadStringCompleted +=new DownloadStringCompletedEventHandler(webClient_DownloadStringCompleted);

            try
            {
                webClient.DownloadStringAsync(new Uri(txtAdress.Text));
            }
            catch (Exception ex)
            {
                Misc.AddErrorLog(ex.Message, txtAdress.Text);
                this.Enabled = true;
                this.Text = Application.ProductName;
            }
        }

        private void webClient_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                if (chkUseCustom.Checked)
                {
                    FindLinksFromCustom(e.Result);
                }
                else
                {
                    FindFullLinks(e.Result);
                    FindPartialLinks(e.Result);
                }

                this.DialogResult = DialogResult.OK;
            }
            else
            {
                Misc.AddErrorLog(e.Error.Message, txtAdress.Text);
            }

            ((WebClient)sender).Dispose();
        }

        private void FindFullLinks(string input)
        {
            string pattern = BuildFullLinkPattern();
            Regex regex = new Regex(pattern, RegexOptions.IgnoreCase);
            MatchCollection MC = regex.Matches(input);

            foreach (Match match in MC)
            {
                // Get the link from the group and convert to lower case if match case is unchecked
                string link = (chkMatchCase.Checked) ? match.Groups["Link"].Value : match.Groups["Link"].Value.ToLower();

                //if (link.Length > 2)
                //{
                //    string startTwo = link.Substring(0, 2);

                //    if (startTwo == "//")
                //    {
                //        link = link.Remove(0, 2);
                //    }
                //}

                if (!chkMatchCase.Checked)
                {
                    txtName.Text = txtName.Text.ToLower();
                }

                if (chkFilterFilename.Checked)
                {
                    if (chkSearchFull.Checked)
                    {
                        if (link.Contains(txtName.Text))
                        {
                            _linksFound.Add(link);
                        }
                    }
                    else
                    {
                        // Get the filename only and check to see if it contains certain text
                        string filename = Path.GetFileName(link);

                        if (filename.Contains(txtName.Text))
                        {
                            _linksFound.Add(link);
                        }
                    }
                }
                else
                {
                    _linksFound.Add(link);
                }
            }
        }

        /// <summary>
        /// Find links from a user defined regex pattern
        /// </summary>
        private void FindLinksFromCustom(string input)
        {
            string pattern = BuildFullLinkPattern();
            Regex regex = new Regex(txtPattern.Text, RegexOptions.IgnoreCase);
            MatchCollection MC = regex.Matches(input);

            foreach (Match match in MC)
            {
                // Get the link from the group and convert to lower case if match case is unchecked
                string link = (chkMatchCase.Checked) ? match.Groups["Link"].Value : match.Groups["Link"].Value.ToLower();

                if (link[0] == '/')
                {
                    _linksFound.Add(CompletePartialLink(link));
                }
                else
                {
                    _linksFound.Add(link);
                }
            }
        }

        private void FindPartialLinks(string input)
        {
            string pattern = BuildPartialLinkPattern();
            Regex regex = new Regex(pattern, RegexOptions.IgnoreCase);
            MatchCollection MC = regex.Matches(input);

            foreach (Match match in MC)
            {
                // Get the link from the group and convert to lower case if match case is unchecked
                string link = (chkMatchCase.Checked) ? match.Groups["Link"].Value : match.Groups["Link"].Value.ToLower();

                if (!chkMatchCase.Checked)
                {
                    txtName.Text = txtName.Text.ToLower();
                }

                if (chkFilterFilename.Checked)
                {
                    if (chkSearchFull.Checked)
                    {
                        if (link.Contains(txtName.Text))
                        {
                            _linksFound.Add(CompletePartialLink(link));
                        }
                    }
                    else
                    {
                        // Get the filename only and check to see if it contains certain text
                        string filename = Path.GetFileName(link);

                        if (filename.Contains(txtName.Text))
                        {
                            _linksFound.Add(CompletePartialLink(link));
                        }
                    }
                }
                else
                {
                    _linksFound.Add(CompletePartialLink(link));
                }
            }
        }

        private string CompletePartialLink(string link)
        {
            string homepage = Path.GetDirectoryName(txtAdress.Text);
            string result = homepage.TrimEnd('\\') + link;

            result = Regex.Replace(result, @"(?<!http:)\\", "/");

            // When using the path method above, the forward slashes are replaced with back slashes so we must fix this
            return result.Replace("\\", "//");
        }

        private string BuildFullLinkPattern()
        {
            string attrib = string.Empty;

            if (chkHref.Checked && chkSrc.Checked)
            {
                attrib = "href|src";
            }
            else if (chkHref.Checked)
            {
                attrib = "href";
            }
            else if (chkSrc.Checked)
            {
                attrib = "src";
            }

            // example pattern (href|src)\s*=\s*\"(?<Link>yourlinkhere.com)\"
            return "(" + attrib + @")=""(?<Link>(https*://|//)[^""]+)""";
        }

        private string BuildPartialLinkPattern()
        {
            string attrib = string.Empty;

            if (chkHref.Checked && chkSrc.Checked)
            {
                attrib = "href|src";
            }
            else if (chkHref.Checked)
            {
                attrib = "href";
            }
            else if (chkSrc.Checked)
            {
                attrib = "src";
            }

            // example pattern (href|src)\s*=\s*\"(?<Link>yourlinkhere.com)\"
            return "(" + attrib + @")=""(?<Link>/[^/][^""]+)""";
        }

        public string[] LinksFound
        {
            get { return _linksFound.ToArray(); }
        }

        private void txtPattern_TextChanged(object sender, EventArgs e)
        {
            groupFilter.Enabled = (txtPattern.TextLength == 0);
        }

        private void btnBuildCurrent_Click(object sender, EventArgs e)
        {
            txtPattern.Text = BuildFullLinkPattern();
        }

        private void chkUseCustom_CheckedChanged(object sender, EventArgs e)
        {
            if (chkUseCustom.Checked)
            {
                txtPattern.Enabled = true;
                groupFilter.Enabled = false;
                txtPattern.Text = BuildFullLinkPattern();
            }
            else
            {
                txtPattern.Enabled = false;
                groupFilter.Enabled = true;
            }
        }
    }
}
